/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#ifndef __PDB_ITEM_H__
#define __PDB_ITEM_H__

#include "pdb.h"
#include "pdbSimpleItem.h"
#include "pdbFile.h"  /*???*/

class pdbItem : public pdbSimpleItem {
public:
  enum access_t { AC_NA, AC_PRIV, AC_PROT, AC_PUB };
  enum store_t { ST_NA, ST_ASM, ST_AUTO, ST_EXT, ST_STAT }; 
  enum func_t { FU_NA, FU_CONV, FU_CTOR, FU_DTOR, FU_OP };
  enum virt_t { VI_NO, VI_PURE, VI_VIRT };
  enum macro_t { MA_NA, MA_DEF, MA_UNDEF };
  enum templ_t { TE_NA, TE_CLASS, TE_FUNC, TE_MEMCLASS, TE_MEMFUNC,
                 TE_STATMEM };
  enum float_t { FL_NA, FL_FLOAT, FL_DBL, FL_LONGDBL };
  enum int_t { I_NA, I_CHAR, I_SCHAR, I_UCHAR, I_SHORT, I_USHORT, I_INT,
               I_UINT, I_LONG, I_ULONG, I_LONGLONG, I_ULONGLONG };
  enum type_t { TY_NA, TY_BOOL, TY_ENUM, TY_ERR, TY_FUNC, TY_VOID, TY_INT,
                TY_FLOAT, TY_PTR, TY_REF, TY_ARRAY, TY_TREF, TY_PTRMEM,
		TY_TPARAM, TY_WCHAR, TY_CLASS };
  enum qual_t { QL_NA, QL_CONST, QL_VOLATILE, QL_RESTRICT };
  enum class_t { CL_NA, CL_CLASS, CL_STRUCT, CL_UNION };
  enum link_t { LK_NA, LK_INTERNAL, LK_CXX, LK_C };
  enum mem_t { M_NA, M_TYPE, M_STATVAR, M_VAR };
  
  pdbItem(int id) : pdbSimpleItem(id), l(),
     acs(AC_NA), cptr(0), nptr(0), fn("") {}
  pdbItem(const string& name, int id) : pdbSimpleItem(name, id),
     l(), acs(AC_NA), cptr(0), nptr(0), fn("") {}
  virtual ~pdbItem() {}

  const string& fullName() const { return fn; }
  access_t access() const { return acs; }
  const pdbClass* parentClass()  const { return cptr; }
  pdbLoc& location() { return l; }
  const pdbLoc& location() const { return l; }
  const pdbNamespace* parentNSpace() const { return nptr; }

  void fullName(const string& name) { fn = name; }
  void access(access_t attr) { acs = attr; }
  void parentClass(const pdbClass* pclass)  { cptr = pclass; }
  void location(const pdbLoc& loc) { l = loc; }
  void parentNSpace(const pdbNamespace* nspace) { nptr = nspace; }
  
  virtual ostream& print(ostream& ostr) const;
  virtual void adjustPtrs(PDB*);
  virtual void process(PDB*);
  virtual dupl_t findDuplicate(pdbSimpleItem* r) {
    pdbItem* rhs = dynamic_cast<pdbItem*> (r);
    //return ( fullName() == rhs->fullName() ? NEWDUPL : NODUPL );

    bool res = fullName() == rhs->fullName();
    if (res) {
      pdbLoc ll = location();
      pdbLoc rl = rhs->location();
      if ( ll.file() && rl.file() &&
         ( (ll.file()->name() != rl.file()->name()) ||
           (ll.line() != rl.line()) || (ll.col() != rl.col()) ) ) {
      cerr << fullName() << " == " << rhs->fullName() << endl;
      cerr << "*** " << ll.file()->name() << " ??? " << rl.file()->name() << endl;
      cerr << "*** " << ll.line() << " ??? " << rl.line() << endl;
      cerr << "*** " << ll.col() << " ??? " << rl.col() << endl;

      }
      return NEWDUPL;
    }
    return NODUPL;
  }

  static const char *toName(access_t v);
  static const char *toName(store_t  v);
  static const char *toName(func_t   v);
  static const char *toName(virt_t   v);
  static const char *toName(macro_t  v);
  static const char *toName(templ_t  v);
  static const char *toName(float_t  v);
  static const char *toName(int_t    v);
  static const char *toName(type_t   v);
  static const char *toName(qual_t   v);
  static const char *toName(class_t  v);
  static const char *toName(link_t   v);
  static const char *toName(mem_t    v);

  static access_t toAccess(const char *v);
  static store_t  toStore(const char *v);
  static func_t   toFunc(const char *v);
  static virt_t   toVirt(const char *v);
  static macro_t  toMacro(const char *v);
  static templ_t  toTempl(const char *v);
  static float_t  toFloat(const char* v);
  static int_t    toInt(const char* v);
  static type_t   toType(const char* v);
  static qual_t   toQual(const char* v);
  static class_t  toClass(const char* v);
  static link_t   toLink(const char* v);
  static mem_t    toMem(const char* v);

private:
  pdbLoc              l;
  access_t            acs;
  const pdbClass     *cptr;
  const pdbNamespace *nptr;
  string              fn;
};

#endif
