/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#ifndef __PDB_SIMPLE_ITEM_H__
#define __PDB_SIMPLE_ITEM_H__

#include <iostream.h>
#include "pdb.h"

class pdbLoc {
public:
  pdbLoc () : fptr(0), li(0), co(0) {}
  pdbLoc (pdbFile *file, int line, int col) : fptr(file), li(line), co(col) {}

  const pdbFile *file() const { return fptr; }
  void file(const pdbFile* file) { fptr = file; }
  int line() const { return li; }
  int col() const { return co; }
  void setLoc(pdbFile *file, int line, int col) {
    fptr = file; li = line; co = col;
  }

  bool operator==(const pdbLoc& rhs) const {
    return file()==rhs.file() && line()==rhs.line() && col()==rhs.col();
  }
  ostream& printLoc(ostream& ostr) const;

private:
  const pdbFile *fptr;
  int li;
  int co;
};

ostream& operator<<(ostream& ostr, const pdbLoc& l);

class pdbSimpleItem {
public:
  enum dupl_t { NODUPL, OLDDUPL, NEWDUPL };

  static const int UNIQUE = INT_MIN+1;
  static const int NOTDEL = INT_MIN;

  pdbSimpleItem(int id) : n(""), i(id), nId(UNIQUE), fl(0) {}
  pdbSimpleItem(const string& name, int id)
               : n(name), i(id), nId(UNIQUE), fl(0) {}

  virtual ~pdbSimpleItem() {}

  const string& name() const { return n; }
  int id() const { return i; }
  int newId() const { return nId; }

  void name(const string& name) { n = name; }
  void id(int ii) { i = ii; }
  void newId(int id) { nId = id; }

  virtual const char *desc() const = 0;
  virtual const char *attrPrefix() const;
  virtual ostream& print(ostream& ostr) const = 0;
  virtual void adjustPtrs(PDB*) = 0;

  virtual void process(PDB*) { /*DO NOTHING*/ }
  virtual dupl_t findDuplicate(pdbSimpleItem* rhs) {
    return ( name() == rhs->name() ? NEWDUPL : NODUPL );
  }

  // simple flag / attribute implementation part i
  int flag() const { return fl; }
  void *attr() const { return at; }
  void flag(int flag) const { fl = flag; }
  void attr(void *attr) const { at = attr; }

private:
  string     n;
  int        i;
  int        nId;

  // simple flag / attribute implementation part ii
  union {
    mutable int      fl;
    mutable void    *at;
  };

};

ostream& operator<<(ostream& ostr, const pdbSimpleItem& i);

#endif
